QTableView::item
{
    qproperty-wordWrap: false;
}

QStatusBar::item {
    padding: 3px;
    border-radius: 3px;
} 

QStatusBar::item {
    padding: 3px;
    border-radius: 3px;
}
QTableWidget::item:selected {
    background-color: #fff9c4;
    color: black;
}


QTableWidget::item:selected {
    background-color: #fff9c4;   /* light yellow */
    color: black;
}

QTableWidget::item:hover {
    background-color: #fffbda;
}

QLineEdit {
    border: 1px solid grey;
    border-radius: 4px; 
    padding: 4px; 
}