

QLineEdit, QTableWidget, QTextEdit, QComboBox, QListWidget
{
background-color: #ffffff;
color: #000000;
}




QPushButton	
{
background-color: #ffffff;
color: #000000;	
border-radius: 2px;
 margin: 2px ;
 padding-left: 8px;
 padding-right: 8px;
 padding-top: 2px;
 padding-bottom: 2px;
 border: 1px solid #cccccc
}


QPushButton:pressed	
{
background-color: #cccccc;
}


/* Style the tab using the tab sub-control. Note that
    it reads QTabBar _not_ QTabWidget */
QTabBar::tab {
    background: white;
    min-width: 8ex;
 padding-left: 8px;
 padding-right: 8px;
 padding-top: 2px;
    color:black;
}

QTabBar::tab:selected, QTabBar::tab:hover {
    background: #3366ff;
    color:white;
}

QTabBar::tab:selected {
    border-color: #3366ff;
    color:white;
}




