IDI_ICON1            ICON                    "pslogo.ico"

#include <windows.h>
#include "globals.h"

VS_VERSION_INFO VERSIONINFO
FILEVERSION     VER_FILEVERSION
PRODUCTVERSION  VER_FILEVERSION
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904E4"
        BEGIN
            VALUE "CompanyName",        "NagleCode, LLC"
            VALUE "FileDescription",    "Network Utility to Send/Receive Packets"
            VALUE "FileVersion",        SW_VERSION
            VALUE "InternalName",       "Packet Sender"
            VALUE "LegalCopyright",     "Copyright (c) NagleCode, LLC"
            VALUE "LegalTrademarks1",   "All Rights Reserved."
            VALUE "LegalTrademarks2",   "All Rights Reserved."
            VALUE "OriginalFilename",   "packetsender.exe"
            VALUE "ProductName",        "Packet Sender"
            VALUE "ProductVersion",     SW_VERSION
        END
    END

    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1252
    END
END
